@@ -267,6 +267,8 @@ impl Network {
         mark_trace!(rpc.trace, served);
     }
 
+    // 8 worker threads can handle the load of 20 clients of KV server
+    // with a 1-second deadline.
     const WORKER_THREADS: usize = 16;
 
     pub fn run_daemon() -> Arc<Mutex<Network>> {
@@ -333,6 +335,7 @@ impl Network {
     fn new() -> (Self, Receiver<Option<RpcOnWire>>) {
         // Server thread pool
         let server_pool = tokio::runtime::Builder::new_multi_thread()
+            .enable_time()
             .thread_name("server-pool")
             .build()
             .expect("Creating server thread pool should not fail");
